@@ -14,20 +14,16 @@ CmdExtract::CmdExtract(CommandData *Cmd)
   TotalFileCount=0;
 
   // Common for all archives involved. Set here instead of DoExtract()
-  // to use in unrar.dll too. Allows to avoid LinksToDirs() calls
-  // and save CPU time in no symlinks including ".." in target were extracted.
-#if defined(_WIN_ALL)
-  // We can't expand symlink path components in another symlink target
-  // in Windows. We can't create symlinks in Android now. Even though we do not
-  // really need LinksToDirs() calls in these systems, we still call it
-  // for extra safety, but only if symlink with ".." in target was extracted.
-  ConvertSymlinkPaths=false;
-#else
+  // to use in unrar.dll too.
   // We enable it by default in Unix to care about the case when several
   // archives are unpacked to same directory with several independent RAR runs.
   // Worst case performance penalty for a lot of small files seems to be ~3%.
+  // 2023.09.15: Windows performance impact seems to be negligible,
+  // less than 0.5% when extracting mix of small files and folders.
+  // So for extra security we enabled it for Windows too, even though
+  // unlike Unix, Windows doesn't expand lnk1 in symlink targets like
+  // "lnk1/../dir", but converts such path to "dir".
   ConvertSymlinkPaths=true;
-#endif
 
   Unp=new Unpack(&DataIO);
 #ifdef RAR_SMP
@@ -38,13 +34,13 @@ CmdExtract::CmdExtract(CommandData *Cmd)
 
 CmdExtract::~CmdExtract()
 {
-  ReleaseAnalyzeData();
+  FreeAnalyzeData();
   delete Unp;
   delete Analyze;
 }
 
 
-void CmdExtract::ReleaseAnalyzeData()
+void CmdExtract::FreeAnalyzeData()
 {
   for (size_t I=0;I<RefList.Size();I++)
   {
@@ -806,6 +802,11 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
             {
               bool UpLink;
               LinkSuccess=ExtractSymlink(Cmd,DataIO,Arc,DestFileName,UpLink);
+
+              // Unix symlink can have its own owner data.
+              if (LinkSuccess)
+                SetFileHeaderExtra(Cmd,Arc,DestFileName);
+              
               ConvertSymlinkPaths|=LinkSuccess && UpLink;
 
               // We do not actually need to reset the cache here if we cache
@@ -846,6 +847,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
             Unp->Init(Arc.FileHead.WinSize,Arc.FileHead.Solid);
             Unp->SetDestSize(Arc.FileHead.UnpSize);
 #ifndef SFX_MODULE
+            // RAR 1.3 - 1.5 archives do not set per file solid flag.
             if (Arc.Format!=RARFMT50 && Arc.FileHead.UnpVer<=15)
               Unp->DoUnpack(15,FileCount>1 && Arc.Solid);
             else
@@ -1466,7 +1468,7 @@ bool CmdExtract::CheckUnpVer(Archive &Arc,const wchar *ArcFileName)
 // 
 void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumbering)
 {
-  ReleaseAnalyzeData(); // If processing non-first archive in multiple archives set.
+  FreeAnalyzeData(); // If processing non-first archive in multiple archives set.
 
   wchar *ArgName=Cmd->FileArgs.GetString();
   Cmd->FileArgs.Rewind();
@@ -1481,10 +1483,16 @@ void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumberi
     wcsncpyz(NextName,ArcName,ASIZE(NextName));
   
   bool MatchFound=false;
-  bool FirstVolume=true;
   bool PrevMatched=false;
   bool OpenNext=false;
 
+  bool FirstVolume=true;
+  
+  // We shall set FirstFile once for all volumes and not for each volume.
+  // So we do not reuse the outdated Analyze->StartPos from previous volume
+  // if extracted file resides completely in the beginning of current one.
+  bool FirstFile=true;
+
   while (true)
   {
     Archive Arc(Cmd);
@@ -1502,7 +1510,6 @@ void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumberi
     }
 
     OpenNext=false;
-    bool FirstFile=true;
     while (Arc.ReadHeader()>0)
     {
       Wait();
@@ -1515,18 +1522,32 @@ void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumberi
       }
       if (HeaderType==HEAD_FILE)
       {
+        if ((Arc.Format==RARFMT14 || Arc.Format==RARFMT15) && Arc.FileHead.UnpVer<=15)
+        {
+          // RAR versions earlier than 2.0 do not set per file solid flag.
+          // They have only the global archive solid flag, so we can't
+          // reliably analyze them here.
+          OpenNext=false;
+          break;
+        }
+
         if (!Arc.FileHead.SplitBefore)
         {
           if (!MatchFound && !Arc.FileHead.Solid) // Can start extraction from here.
           {
             // We would gain nothing and unnecessarily complicate extraction
-            // by setting these values for first volume or first archived file.
+            // if we set StartName for first volume or StartPos for first
+            // archived file.
             if (!FirstVolume)
               wcsncpyz(Analyze->StartName,NextName,ASIZE(Analyze->StartName));
+
+            // We shall set FirstFile once for all volumes for this code
+            // to work properly. Alternatively we could append 
+            // "|| Analyze->StartPos!=0" to the condition, so we do not reuse
+            // the outdated Analyze->StartPos value from previous volume.
             if (!FirstFile)
               Analyze->StartPos=Arc.CurBlockPos;
           }
-          FirstFile=false;
 
           if (Cmd->IsProcessFile(Arc.FileHead,NULL,MATCH_WILDSUBPATH,0,NULL,0)!=0)
           {
@@ -1571,16 +1592,18 @@ void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumberi
           {
             if (PrevMatched) // First non-matched item after matched.
             {
-              // We would gain nothing and unnecessarily complicate extraction
-              // by setting these values for first volume or first archived file.
+              // We would perform the unnecessarily string comparison
+              // when extracting if we set this value for first volume
+              // or non-volume archive.
               if (!FirstVolume)
                 wcsncpyz(Analyze->EndName,NextName,ASIZE(Analyze->EndName));
-              if (!FirstFile)
-                Analyze->EndPos=Arc.CurBlockPos;
+              Analyze->EndPos=Arc.CurBlockPos;
             }
             PrevMatched=false;
           }
         }
+
+        FirstFile=false;
         if (Arc.FileHead.SplitAfter)
         {
           OpenNext=true; // Allow open next volume.
@@ -1595,6 +1618,12 @@ void CmdExtract::AnalyzeArchive(const wchar *ArcName,bool Volume,bool NewNumberi
     {
       NextVolumeName(NextName,ASIZE(NextName),!Arc.NewNumbering);
       FirstVolume=false;
+
+      // Needed for multivolume archives. Added in case some 'break'
+      // will quit early from loop above, so we do not set it in the loop.
+      // Now it can happen for hypothetical archive without file records
+      // and with HEAD_ENDARC record.
+      FirstFile=false;
     }
     else
       break;
